###################################################
#                                                 #
#   Week 1 - August 24, 2009 - Lyrics Viewer      #
#                                                 #
#                Ruby on Rails                    #
#                                                 #
#               by Rob Williams                   #
#                                                 #
###################################################

require 'hpricot'
require 'rubygems'
require 'open-uri'

class LyricsViewerController < ApplicationController
  
  def index
    # index action shows the form
  end
  
  def fetch
    artist = params[:lyric][:artist]
    song = params[:lyric][:song]
    
    # put artist and song in the format required by Lyrics Wiki. E.g., "miley cyrus" becomes "Miley_Cyrus"
    artist = format_input_for_lyricswiki(artist)
    song = format_input_for_lyricswiki(song)
    url = "http://lyricwiki.org/" + artist + ":" + song
    metro_url = "http://www.metrolyrics.com/" + song + "-lyrics-" + artist + ".html"
    
    # Lyrics Wiki fetch -- easier and better
    begin
      doc = open(url) { |f| Hpricot(f) }
      render :text => "<h3>" + (doc/"h1.firstHeading").inner_html + "</h3><p>" + (doc/"div.lyricbox").inner_html + "</p>"
    rescue OpenURI::HTTPError => e
      # if lyricswiki failed, try Metro Lyrics
      begin
        doc = open(metro_url) { |f| Hpricot(f) }
        render :text => "<h3>" + (doc/"div#bgBoxLyrics h4").inner_html + "</h3><p>" + (doc/"span#lyrics").inner_html + "</p>"
      rescue OpenURI::HTTPError => e
        render :text => "There was a problem loading the lyrics page. Check the artist and song name."
      end
    end
    
  end
  
private

  def format_input_for_lyricswiki(str)
    str.split(/\s+/).each{ |word| word.capitalize! }.join('_') # convert whitespace to _ and capitalize letters
  end
  
  def format_input_for_metrolyrics(str)
    str.split(/\s+/).each{ |word| word.downcase! }.join('-') # convert whitespace to _ and capitalize letters
  end
  
  # Metrolyrics stores their lyrics encoded as their ASCII values, in a failed attempt to protect their lyrics
  # for some unknown reason. This takes an encoded string and returns the decoded lyrics.
  # Note: This isn't even necessary because the browser can decode the ASCII automatically, but this was used to debug
  #       the fetching. 
  def metrolyrics_decode(lyrics)
    ret_str = ""
    array = lyrics.scan(/(\D+)(\d+)/)
    array.each do |x|
      # x[0] is the non-digit string matched
      # x[1] is the digit string matched (the ASCII code)
      ret_str += "\n" if x[0] =~ /br/
      ret_str += x[1].to_i.chr # output actual character, not code
    end
    ret_str += "\n"
  end
  
end
